@@ -16,10 +16,12 @@ import qualified KAT_MiyaguchiPreneel
 import qualified KAT_CMAC
 import qualified KAT_HMAC
 import qualified KAT_HKDF
+import qualified KAT_Argon2
 import qualified KAT_PBKDF2
 import qualified KAT_Curve25519
-import qualified KAT_Ed448
+import qualified KAT_Curve448
 import qualified KAT_Ed25519
+import qualified KAT_OTP
 import qualified KAT_PubKey
 import qualified KAT_Scrypt
 -- symmetric cipher --------------------
@@ -29,6 +31,7 @@ import qualified KAT_Camellia
 import qualified KAT_DES
 import qualified KAT_RC4
 import qualified KAT_TripleDES
+import qualified KAT_Twofish
 -- misc --------------------------------
 import qualified KAT_AFIS
 import qualified Padding
@@ -47,14 +50,16 @@ tests = testGroup "cryptonite"
         , KAT_HMAC.tests
         ]
     , KAT_Curve25519.tests
-    , KAT_Ed448.tests
+    , KAT_Curve448.tests
     , KAT_Ed25519.tests
     , KAT_PubKey.tests
+    , KAT_OTP.tests
     , testGroup "KDF"
         [ KAT_PBKDF2.tests
         , KAT_Scrypt.tests
         , BCrypt.tests
         , KAT_HKDF.tests
+        , KAT_Argon2.tests
         ]
     , testGroup "block-cipher"
         [ KAT_AES.tests
@@ -62,6 +67,7 @@ tests = testGroup "cryptonite"
         , KAT_Camellia.tests
         , KAT_DES.tests
         , KAT_TripleDES.tests
+        , KAT_Twofish.tests
         ]
     , testGroup "stream-cipher"
         [ KAT_RC4.tests
